"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const port_default = 3000;
const rest = require("./rest_client");
//////////////////////////////////////////////////////////////////////
// Storage-coordination implementation
class HttpStorage {
    constructor(port) {
        this.port = port || port_default;
        this.client = rest.create({});
        this.url_prefix = "http://localhost:" + this.port + "/";
    }
    getProperties(callback) {
        const res = [];
        res.push({ key: "type", value: "HttpCoordinator" });
        res.push({ key: "port", value: this.port });
        res.push({ key: "url_prefix", value: this.url_prefix });
        callback(null, res);
    }
    getMessages(name, callback) {
        this.call("get-messages", { worker: name }, callback);
    }
    getMessage(name, callback) {
        this.call("get-message", { worker: name }, callback);
    }
    getWorkerStatus(callback) {
        this.call("worker-statuses", {}, callback);
    }
    getTopologyStatus(callback) {
        this.call("topology-statuses", {}, callback);
    }
    getTopologiesForWorker(name, callback) {
        this.call("worker-topologies", { worker: name }, callback);
    }
    getTopologyInfo(uuid, callback) {
        this.call("topology-info", { uuid }, callback);
    }
    registerWorker(name, callback) {
        this.call("register-worker", { worker: name }, callback);
    }
    pingWorker(name, callback) {
        this.call("ping-worker", { worker: name }, callback);
    }
    announceLeaderCandidacy(name, callback) {
        this.call("announce-leader-candidacy", { worker: name }, callback);
    }
    checkLeaderCandidacy(name, callback) {
        this.call("check-leader-candidacy", { name }, callback);
    }
    assignTopology(uuid, name, callback) {
        this.call("assign-topology", { name, uuid }, callback);
    }
    sendMessageToWorker(worker, cmd, content, valid_msec, callback) {
        this.call("send-message", { worker, cmd, content, valid_msec }, callback);
    }
    setTopologyStatus(uuid, worker, status, error, callback) {
        this.call("set-topology-status", { uuid, status, error, worker }, callback);
    }
    getMsgQueueContent(callback) {
        this.call("get-msg-queue-content", {}, callback);
    }
    setWorkerStatus(name, status, callback) {
        this.call("set-worker-status", { name, status }, callback);
    }
    setWorkerLStatus(name, lstatus, callback) {
        this.call("set-worker-lstatus", { name, lstatus }, callback);
    }
    setTopologyPid(uuid, pid, callback) {
        this.call("set-topology-pid", { uuid, pid }, callback);
    }
    registerTopology(uuid, config, callback) {
        this.call("register-topology", { uuid, config }, callback);
    }
    disableTopology(uuid, callback) {
        this.call("disable-topology", { uuid }, callback);
    }
    enableTopology(uuid, callback) {
        this.call("enable-topology", { uuid }, callback);
    }
    deleteTopology(uuid, callback) {
        this.call("delete-topology", { uuid }, callback);
    }
    clearTopologyError(uuid, callback) {
        this.call("clear-topology-error", { uuid }, callback);
    }
    stopTopology(uuid, callback) {
        this.call("stop-topology", { uuid }, callback);
    }
    killTopology(uuid, callback) {
        this.call("kill-topology", { uuid }, callback);
    }
    deleteWorker(name, callback) {
        this.call("delete-worker", { name }, callback);
    }
    shutDownWorker(name, callback) {
        this.call("shut-down-worker", { name }, callback);
    }
    getTopologyHistory(uuid, callback) {
        this.call("topology-history", { uuid }, callback);
    }
    getWorkerHistory(name, callback) {
        this.call("worker-history", { name }, callback);
    }
    call(addr, req_data, callback) {
        const self = this;
        this.client.post(self.url_prefix + addr, req_data)
            .then(res => callback(null, res.data))
            .catch(err => callback(err));
    }
}
exports.HttpStorage = HttpStorage;
//# sourceMappingURL=http_storage.js.map