"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const async = require("async");
const pm = require("../util/pattern_matcher");
/** This bolt routs incoming messages based on provided
 * queries and sends them forward using mapped stream ids.
 */
class RouterBolt {
    /** Simple constructor */
    constructor() {
        this.onEmit = null;
        this.matchers = [];
    }
    /** Initializes routing patterns */
    init(name, config, context, callback) {
        this.onEmit = config.onEmit;
        for (const stream_id in config.routes) {
            if (config.routes.hasOwnProperty(stream_id)) {
                const filter = config.routes[stream_id];
                this.matchers.push({
                    matcher: new pm.PaternMatcher(filter),
                    stream_id
                });
            }
        }
        callback();
    }
    heartbeat() {
        // no-op
    }
    shutdown(callback) {
        callback();
    }
    receive(data, stream_id, callback) {
        const tasks = [];
        for (const item of this.matchers) {
            if (item.matcher.isMatch(data)) {
                /* jshint loopfunc:true */
                tasks.push(xcallback => {
                    this.onEmit(data, item.stream_id, xcallback);
                });
            }
        }
        async.parallel(tasks, callback);
    }
}
exports.RouterBolt = RouterBolt;
//# sourceMappingURL=router_bolt.js.map