"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rest = require("../distributed/http_based/rest_client");
const log = require("../util/logger");
/** This spout periodically checks specified RSS feed and emits all items. */
class RssSpout {
    constructor() {
        this.tuples = [];
        this.should_run = false;
    }
    init(name, config, context, callback) {
        this.name = name;
        this.logging_prefix = `[RssSpout ${this.name}] `;
        this.stream_id = config.stream_id;
        this.url = config.url;
        this.repeat = config.repeat || 10 * 60 * 1000;
        this.next_call_after = Date.now() - 10;
        this.client = rest.create({});
        callback();
    }
    heartbeat() {
        if (Date.now() >= this.next_call_after && this.should_run) {
            log.logger().debug(this.logging_prefix + "Starting RSS crawl: " + this.url);
            this.client.get(this.url)
                .then(res => {
                for (const item of res.data.rss.channel.item) {
                    this.tuples.push(item);
                }
                this.next_call_after = Date.now() + this.repeat;
            })
                .catch(() => {
                // do nothing
            });
        }
    }
    shutdown(callback) {
        this.should_run = false;
        callback();
    }
    run() {
        this.should_run = true;
    }
    pause() {
        this.should_run = false;
    }
    next(callback) {
        if (!this.should_run) {
            return callback(null, null, null);
        }
        if (this.tuples.length == 0) {
            return callback(null, null, null);
        }
        const data = this.tuples[0];
        this.tuples = this.tuples.slice(1);
        callback(null, data, this.stream_id);
    }
}
exports.RssSpout = RssSpout;
//# sourceMappingURL=rss_spout.js.map