"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/** Wrapper for async bolt that transforms it into normal, callback-based bolt */
class BoltAsyncWrapper {
    constructor(obj) {
        this.inner = obj;
    }
    init(name, config, context, callback) {
        const new_config = Object.assign({}, config);
        new_config.onEmit = (data, stream_id) => {
            return new Promise((resolve, reject) => {
                config.emit(data, stream_id, (err) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve();
                    }
                });
            });
        };
        this.inner.init(name, config, context)
            .then(() => { callback(); })
            .catch(callback);
    }
    heartbeat() { this.inner.heartbeat(); }
    shutdown(callback) {
        this.inner.shutdown()
            .then(() => { callback(); })
            .catch(callback);
    }
    receive(data, stream_id, callback) {
        this.inner.receive(data, stream_id)
            .then(() => { callback(); })
            .catch(callback);
    }
}
exports.BoltAsyncWrapper = BoltAsyncWrapper;
/** Wrapper for async spout that transforms it into normal, callback-based spout */
class SpoutAsyncWrapper {
    constructor(obj) {
        this.inner = obj;
    }
    init(name, config, context, callback) {
        this.inner.init(name, config, context)
            .then(() => { callback(); })
            .catch(callback);
    }
    shutdown(callback) {
        this.inner.shutdown()
            .then(() => { callback(); })
            .catch(callback);
    }
    heartbeat() { this.inner.heartbeat(); }
    run() { this.inner.run(); }
    pause() { this.inner.pause(); }
    next(callback) {
        this.inner.next()
            .then(res => {
            if (res) {
                callback(null, res.data, res.stream_id);
            }
            else {
                callback(null, null, null);
            }
        })
            .catch(err => { callback(err, null, null); });
    }
}
exports.SpoutAsyncWrapper = SpoutAsyncWrapper;
//# sourceMappingURL=topology_async_wrappers.js.map