"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function parseCommandLine(argv) {
    const res = { _: [] };
    let last_switch = null;
    for (const curr of argv) {
        if (curr.slice(0, 2) == "--") {
            last_switch = curr.slice(2);
            res[last_switch] = true;
        }
        else if (curr.slice(0, 1) == "-") {
            last_switch = curr.slice(1);
            res[last_switch] = true;
        }
        else if (last_switch) {
            res[last_switch] = curr;
            last_switch = null;
        }
        else {
            res._.push(curr);
        }
    }
    return res;
}
exports.parseCommandLine = parseCommandLine;
function parseCommandLineEx(argv, map) {
    map = map || {};
    const res = parseCommandLine(argv);
    for (const field in map) {
        if (map.hasOwnProperty(field) && res[field]) {
            res[map[field]] = res[field];
        }
    }
    return res;
}
exports.parseCommandLineEx = parseCommandLineEx;
class OptionsDescription {
}
exports.OptionsDescription = OptionsDescription;
class CmdLineParser {
    constructor() {
        this.shortnames = new Map();
        this.names = new Map();
        this.descriptions = [];
    }
    clear() {
        this.shortnames.clear();
        this.names.clear();
        this.descriptions = [];
    }
    areFlags(letters) {
        for (const letter of letters) {
            if (!this.shortnames[letter] || !this.shortnames[letter].flag) {
                return false;
            }
        }
        return true;
    }
    getValue(text) {
        if (!text) {
            return null;
        }
        for (const letter of text) {
            if (letter < "0" || letter > "9") {
                return text;
            }
        }
        return parseInt(text, 10);
    }
    getTargetName(name, description) {
        if (!description) {
            return name;
        }
        if (description.target) {
            return description.target;
        }
        return description.name;
    }
    define(shortname, name, defaultValue, text, options) {
        options = options || {};
        const description = {
            default: defaultValue,
            flag: null,
            name,
            shortname,
            target: null,
            text
        };
        if (options.name) {
            description.target = options.name;
        }
        if (options.flag) {
            description.flag = true;
        }
        this.descriptions.push(description);
        this.names[name] = description;
        this.shortnames[shortname] = description;
        return this;
    }
    process(args) {
        const opts = {};
        this.descriptions.forEach(description => {
            if (description.default) {
                opts[this.getTargetName(null, description)] = description.default;
            }
        });
        for (let k = 0; k < args.length; k++) {
            const arg = args[k];
            if (arg.length > 2 && arg[0] == "-" && arg[1] == "-") {
                const name = arg.slice(2);
                const description = this.names[name];
                if (description && description.flag) {
                    opts[this.getTargetName(name, description)] = true;
                }
                else {
                    k++;
                    const val = this.getValue(args[k]);
                    opts[this.getTargetName(name, description)] = val;
                }
            }
            else if (arg.length > 1 && arg[0] == "-") {
                const shortname = arg.slice(1);
                let description = this.shortnames[shortname];
                if (!description && this.areFlags(shortname)) {
                    for (let j = 0; j < shortname.length; j++) {
                        const letter = this.shortnames[j];
                        description = this.shortnames[letter];
                        opts[this.getTargetName(letter, description)] = true;
                    }
                    continue;
                }
                if (description && description.flag) {
                    opts[this.getTargetName(shortname, description)] = true;
                }
                else {
                    k++;
                    const val = this.getValue(args[k]);
                    opts[this.getTargetName(shortname, description)] = val;
                }
            }
        }
        return opts;
    }
}
exports.CmdLineParser = CmdLineParser;
//# sourceMappingURL=cmdline.js.map